@@ -12,17 +12,33 @@ Below are some notes on how they are stored.
 import inspect
 import sys
 import typing
-from types import CodeType
+from types import CodeType, FunctionType, ModuleType
 from typing import Any, Callable, List, Optional, Set, Tuple
 
 import wrapt
 from typeguard import check_type
 
+# Configuration options
+
+ENABLE_CONTRACT_CHECKING = True
+"""
+Set to True to enable contract checking.
+"""
+
 DEBUG_CONTRACTS = False
 """
 Set to True to display debugging messages when checking contracts.
 """
 
+RENAME_MAIN_TO_PYDEV_UMD = True
+"""
+Set to False to disable workaround for PyCharm's "Run File in Python Console" action.
+In most cases you should not need to change this!
+"""
+
+_PYDEV_UMD_NAME = "pydev_umd"
+
+
 _DEFAULT_MAX_VALUE_LENGTH = 30
 FUNCTION_RETURN_VALUE = "$return_value"
 
@@ -42,11 +58,18 @@ def check_all_contracts(*mod_names: str, decorate_main: bool = True) -> None:
         decorate_main: True if the module being run (where __name__ == '__main__') should
             have contracts checked.
     """
+    if not ENABLE_CONTRACT_CHECKING:
+        return
 
     modules = []
     if decorate_main:
         mod_names = mod_names + ("__main__",)
 
+        # Also add _PYDEV_UMD_NAME, handling when the file is being run in PyCharm
+        # with the "Run in Python Console" action.
+        if RENAME_MAIN_TO_PYDEV_UMD:
+            mod_names = mod_names + (_PYDEV_UMD_NAME,)
+
     for module_name in mod_names:
         modules.append(sys.modules.get(module_name, None))
 
@@ -63,7 +86,7 @@ def check_all_contracts(*mod_names: str, decorate_main: bool = True) -> None:
 def _enable_function_contracts(wrapped, instance, args, kwargs):
     """A decorator that enables checking contracts for a function."""
     try:
-        if instance and inspect.isclass(instance):
+        if instance is not None and inspect.isclass(instance):
             # This is a class method, so there is no instance.
             return _check_function_contracts(wrapped, None, args, kwargs)
         else:
@@ -90,6 +113,9 @@ def check_contracts(func_or_class: Any, module_names: Optional[Set[str]] = None)
         ...     \"\"\"
         ...     return x // y
     """
+    if not ENABLE_CONTRACT_CHECKING:
+        return func_or_class
+
     if module_names is not None and func_or_class.__module__ not in module_names:
         _debug(
             f"Warning: skipping contract check for {func_or_class.__name__} defined in {func_or_class.__module__} because module is not included as an argument."
@@ -107,7 +133,7 @@ def check_contracts(func_or_class: Any, module_names: Optional[Set[str]] = None)
 
 def add_class_invariants(klass: type) -> None:
     """Modify the given class to check representation invariants and method contracts."""
-    if "__representation_invariants__" in klass.__dict__:
+    if not ENABLE_CONTRACT_CHECKING or "__representation_invariants__" in klass.__dict__:
         # This means the class has already been decorated
         return
 
@@ -138,7 +164,8 @@ def add_class_invariants(klass: type) -> None:
 
         Check representation invariants for this class when not within an instance method of the class.
         """
-        cls_annotations = typing.get_type_hints(klass)
+        klass_mod = _get_module(klass)
+        cls_annotations = typing.get_type_hints(klass, globalns=klass_mod.__dict__)
 
         if name in cls_annotations:
             try:
@@ -156,8 +183,8 @@ def add_class_invariants(klass: type) -> None:
         frame_locals = callframe[1].frame.f_locals
         if self is not frame_locals.get("self"):
             # Only validating if the attribute is not being set in a instance/class method
-            klass_mod = sys.modules.get(klass.__module__)
-            if klass_mod is not None:
+            klass_mod = _get_module(klass)
+            if klass_mod is not None and ENABLE_CONTRACT_CHECKING:
                 try:
                     _check_invariants(self, klass, klass_mod.__dict__)
                 except PyTAContractError as e:
@@ -176,7 +203,11 @@ def add_class_invariants(klass: type) -> None:
 
 def _check_function_contracts(wrapped, instance, args, kwargs):
     params = wrapped.__code__.co_varnames[: wrapped.__code__.co_argcount]
-    annotations = typing.get_type_hints(wrapped)
+    if instance is not None:
+        klass_mod = _get_module(type(instance))
+        annotations = typing.get_type_hints(wrapped, globalns=klass_mod.__dict__)
+    else:
+        annotations = typing.get_type_hints(wrapped)
     args_with_self = args if instance is None else (instance,) + args
 
     # Check function parameter types
@@ -184,7 +215,7 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
         if param in annotations:
             try:
                 _debug(f"Checking type of parameter {param} in call to {wrapped.__qualname__}")
-                check_type(param, arg, annotations[param])
+                check_type_strict(param, arg, annotations[param])
             except TypeError:
                 additional_suggestions = _get_argument_suggestions(arg, annotations[param])
 
@@ -216,7 +247,8 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
                 continue
             target.__preconditions__.append((precondition, compiled))
 
-    _check_assertions(wrapped, function_locals)
+    if ENABLE_CONTRACT_CHECKING:
+        _check_assertions(wrapped, function_locals)
 
     # Check return type
     r = wrapped(*args, **kwargs)
@@ -224,7 +256,7 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
         return_type = annotations["return"]
         try:
             _debug(f"Checking return type from call to {wrapped.__qualname__}")
-            check_type("return", r, return_type)
+            check_type_strict("return", r, return_type)
         except TypeError:
             raise PyTAContractError(
                 f"{wrapped.__name__}'s return value {_display_value(r)} did not match "
@@ -249,16 +281,32 @@ def _check_function_contracts(wrapped, instance, args, kwargs):
                 continue
             target.__postconditions__.append((postcondition, compiled, return_val_var_name))
 
-    _check_assertions(
-        wrapped,
-        function_locals,
-        function_return_val=r,
-        condition_type="postcondition",
-    )
+    if ENABLE_CONTRACT_CHECKING:
+        _check_assertions(
+            wrapped,
+            function_locals,
+            function_return_val=r,
+            condition_type="postcondition",
+        )
 
     return r
 
 
+def check_type_strict(argname: str, value: Any, expected_type: type) -> None:
+    """Ensure that ``value`` matches ``expected_type``.
+
+    Differentiates between:
+        - float vs. int
+        - bool vs. int
+    """
+    if ENABLE_CONTRACT_CHECKING:
+        if (type(value) is int and expected_type is float) or (
+            type(value) is bool and expected_type is int
+        ):
+            raise TypeError(f"type of {argname} must be {expected_type}; got {value} instead")
+        check_type(argname, value, expected_type)
+
+
 def _get_argument_suggestions(arg: Any, annotation: type) -> str:
     """Returns potential suggestions for the given arg and its annotation"""
     try:
@@ -278,8 +326,8 @@ def _instance_method_wrapper(wrapped: Callable, klass: type) -> Callable:
             if _instance_init_in_callstack(instance):
                 return r
             _check_class_type_annotations(klass, instance)
-            klass_mod = sys.modules.get(klass.__module__)
-            if klass_mod is not None:
+            klass_mod = _get_module(klass)
+            if klass_mod is not None and ENABLE_CONTRACT_CHECKING:
                 _check_invariants(instance, klass, klass_mod.__dict__)
         except PyTAContractError as e:
             raise AssertionError(str(e)) from None
@@ -316,7 +364,8 @@ def _check_class_type_annotations(klass: type, instance: Any) -> None:
     Precondition:
         - isinstance(instance, klass)
     """
-    cls_annotations = typing.get_type_hints(klass)
+    klass_mod = _get_module(klass)
+    cls_annotations = typing.get_type_hints(klass, globalns=klass_mod.__dict__)
 
     for attr, annotation in cls_annotations.items():
         value = getattr(instance, attr)
@@ -345,7 +394,16 @@ def _check_invariants(instance, klass: type, global_scope: dict) -> None:
             _debug(f"Warning: could not evaluate representation invariant: {invariant}")
         else:
             if not check:
-                raise PyTAContractError(f'Representation invariant "{invariant}" was violated')
+                curr_attributes = ", ".join(
+                    f"{k}: {_display_value(v)}" for k, v in vars(instance).items()
+                )
+
+                curr_attributes = "{" + curr_attributes + "}"
+
+                raise PyTAContractError(
+                    f'"{instance.__class__.__name__}" representation invariant "{invariant}" was violated for'
+                    f" instance attributes {curr_attributes}"
+                )
 
 
 def _get_legal_return_val_var_name(var_dict: dict) -> str:
@@ -499,6 +557,38 @@ def _display_annotation(annotation: Any) -> str:
         return repr(annotation)
 
 
+def _get_module(obj: Any) -> ModuleType:
+    """Return the module where obj was defined (normally obj.__module__).
+
+    NOTE: this function defines a special case when using PyCharm and the file
+    defining the object is "Run in Python Console". In this case, the pydevd runner
+    renames the '__main__' module to 'pydev_umd', and so we need to access that
+    module instead. This behaviour can be disabled by setting RENAME_MAIN_TO_PYDEV_UMD
+    to False.
+    """
+    module_name = obj.__module__
+    module = sys.modules[module_name]
+
+    if (
+        module_name != "__main__"
+        or not RENAME_MAIN_TO_PYDEV_UMD
+        or _PYDEV_UMD_NAME not in sys.modules
+    ):
+        return module
+
+    # Get a function/class name to check whether it is defined in the module
+    if isinstance(obj, (FunctionType, type)):
+        name = obj.__name__
+    else:
+        # For any other type of object, be conservative and just return the module
+        return module
+
+    if name in vars(module):
+        return module
+    else:
+        return sys.modules[_PYDEV_UMD_NAME]
+
+
 def _debug(msg: str) -> None:
     """Display a debugging message.
 
